/*
    Copyright (c) 2010, Nick Sotnichek
    All rights reserved.

    Redistribution and use in source and binary forms, with or without
    modification, are permitted provided that the following conditions are met:
        * Redistributions of source code must retain the above copyright
        notice, this list of conditions and the following disclaimer.
        * Redistributions in binary form must reproduce the above copyright
        notice, this list of conditions and the following disclaimer in the
        documentation and/or other materials provided with the distribution.
        * Neither the name of the author nor the
        names of its contributors may be used to endorse or promote products
        derived from this software without specific prior written permission.

    THIS SOFTWARE IS PROVIDED BY NICK SOTNICHEK ''AS IS'' AND ANY
    EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
    WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
    DISCLAIMED. IN NO EVENT SHALL NICK SOTNICHEK BE LIABLE FOR ANY
    DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
    (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
    LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
    ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*/

#include "countryselector.h"
#include <qfile.h>
#include <qtextstream.h>
#include <qstring.h>
#include <kcombobox.h>

CountrySelector::CountrySelector(QGraphicsWidget* parent): QObject()
{
  this->parent=parent;
  filllist();
  initFrame();
}

CountrySelector::~CountrySelector()
{

}

void CountrySelector::filllist()
{
  QFile file(":/iso3166-countrycodes.txt");
  QFile filep(":/postal_available.txt");
  int first_str=12;
  //int name_col=0;
  int a2_col=48;
  int a3_col=56;
  int num_col=64;
  
  if (!file.open(QIODevice::ReadOnly | QIODevice::Text))
         return;
  
  QList <QString> pa;
  
  if(filep.open(QIODevice::ReadOnly | QIODevice::Text))
  {
      QTextStream in(&filep);
      while (!in.atEnd())   
      {
	  QString line = in.readLine();
	  line=line.trimmed();
	  pa.append(line);
      }
  }
  
  int i=0;
  QTextStream in(&file);
  while (!in.atEnd()) 
  {
    QString line = in.readLine();
    if(++i<first_str)
      continue;    
    country c1;
    c1.name=line.left(a2_col);
    c1.name=c1.name.trimmed();
    c1.a2=line.mid(a2_col,a3_col-a2_col);
    c1.a2=c1.a2.trimmed();
    c1.a3=line.mid(a3_col,num_col-a3_col);
    c1.a3=c1.a3.trimmed(); 
    QString s=line.mid(num_col);
    c1.number=s.toInt();
    if(pa.indexOf(c1.a3)>=0)
      c1.postal=true;
    else
      c1.postal=false;
    countrylist.append(c1);
  }
}

void CountrySelector::initFrame()
{
	int fsize=11;
	QFont font9;
        font9.setPixelSize (fsize);

	QFont font7;
        font7.setPixelSize (9);	
	
	mFrame = new Plasma::Frame(parent);
	mFrame->setOpacity(0.9);
	mFrame->setFrameShadow(Plasma::Frame::Raised);
//	QString style="backgroundcolor: rgb(86, 89, 68);";
//	mFrame->setStyleSheet(style);
	mFrame -> setVisible(false);
//	mFrame -> setVisible(true);
	mFrame->setZValue(20);	
//	mFrame->resize(213, 98);

	mOk = new Plasma::PushButton(mFrame);
	mOk->setMinimumHeight(5);
        mOk->setGeometry(QRect(20, 67, 71, 25));
	mOk->setFont(font9);
	
        mCancel = new Plasma::PushButton(mFrame);
	mCancel->setMinimumHeight(5);
        mCancel->setGeometry(QRect(122, 67, 71, 25));
	mCancel->setFont(font9);
	
	mCBox = new Plasma::ComboBox(mFrame);
        mCBox->setGeometry(QRect(100, 10, 121, 22));
	mCBox->setMaximumWidth(121);
	mCBox->setFont(font7);	
		
	mLabel1 = new Plasma::Label(mFrame);
	mLabel1->setGeometry(QRect(15, 12, 91, 16));
	mLabel1->setFont(font9);
//	mLabel1->setMinimumHeight(5);
//	mLabel1->setMaximumHeight(40);
//	mLabel1->setMaximumWidth(200);
//	mLabel1->setMinimumWidth(5);
//	mLabel1->setGeometry(QRect(40, 40, 120, 30));	
	
	mLabel2 = new Plasma::Label(mFrame);
	mLabel2->setGeometry(QRect(15, 42, 81, 16));
	mLabel2->setFont(font9);
	
	mLEdit = new Plasma::LineEdit(mFrame);
	mLEdit -> setGeometry(QRect(100, 37, 121, 20));
	mLEdit -> setMaximumWidth(121);
	mLEdit ->setFont(font9);
	
	mOk->setText(tr("Ok"));
	mCancel->setText(tr("Cancel"));
	mLabel1->setText(tr("Select country"));
	mLabel2->setText(tr("Postal code"));
	
//	mOk->setVisible(false);
//	mCancel->setVisible(false);
//	mCBox->setVisible(false);
//	mLabel1->setVisible(false);
//	mLabel2->setVisible(false);
//	mLEdit->setVisible(false);

	for(int i=0;i<countrylist.size();i++)
	{
	  country c=countrylist[i];
	  mCBox->addItem(c.name+QString(" (")+c.a2+QString(")"));

	}
	
//	QString s;
//	s=s.setNum(countrylist.size());
//	mLEdit->setText(s);

	connect(mCancel,SIGNAL(clicked()),this,SLOT(Cancel()));
	connect(mOk,SIGNAL(clicked()),this,SLOT(Ok()));
	connect(mCBox,SIGNAL(textChanged(const QString &)),this,SLOT(CountryChanged(const QString &)));
	connect(mLEdit,SIGNAL(textChanged(const QString &)),this,SLOT(PostalChanged(const QString &)));
}

void CountrySelector::Cancel()
{
  emit CountryCanceled();
}

void CountrySelector::Ok()
{
  int i=static_cast<KComboBox*>(mCBox->widget())->currentIndex();
  QString resp="";
  if(countrylist[i].postal)
    resp=s_postal;
  emit CountrySelected(countrylist[i].a2.toLower(),resp);
}

void CountrySelector::paintInterface(QPainter *p, const QStyleOptionGraphicsItem *option, const QRect &contentsRect)
{
  
}

void CountrySelector::PostalChanged(const QString& text)
{
    if(text.length()>0)
      s_postal=text;
}

void CountrySelector::CountryChanged(const QString& text)
{
    int i=static_cast<KComboBox*>(mCBox->widget())->currentIndex();
    if(countrylist[i].postal)
    {
      mLEdit->setText(s_postal);
      mLEdit->setEnabled(true);
    }
    else
    {
      mLEdit->setText("");
      mLEdit->setEnabled(false);
    }
}


void CountrySelector::setCountry(QString ss, QString cs)
{
    s_country=ss;
    s_postal=cs;
    int i=0;
    for(i=0;i<countrylist.size();i++)
    {
	  country c=countrylist[i];
	  if(c.a2.compare(ss,Qt::CaseInsensitive)==0)
	    break;
    }    
    static_cast<KComboBox*>(mCBox->widget())->setCurrentIndex(i);
    if(countrylist[i].postal)  
    {
	mLEdit->setText(s_postal);
	mLEdit->setEnabled(true);
    }
    else
    {
	mLEdit->setText("");
	mLEdit->setEnabled(false);      
    }
}

void CountrySelector::setGeomerty(QRect rect)
{
	    mFrame->setGeometry(rect);
	    mFrame->setMinimumHeight(5);
	    mFrame->setMaximumHeight(500);	    
	    mFrame->setGeometry(rect);	    
}

void CountrySelector::setVisible(bool visible)
{
    		mFrame->setVisible(visible);
		this->visible = visible;
}
