/*
    Copyright (c) 2010, Nick Sotnichek
    All rights reserved.

    Redistribution and use in source and binary forms, with or without
    modification, are permitted provided that the following conditions are met:
        * Redistributions of source code must retain the above copyright
        notice, this list of conditions and the following disclaimer.
        * Redistributions in binary form must reproduce the above copyright
        notice, this list of conditions and the following disclaimer in the
        documentation and/or other materials provided with the distribution.
        * Neither the name of the author nor the
        names of its contributors may be used to endorse or promote products
        derived from this software without specific prior written permission.

    THIS SOFTWARE IS PROVIDED BY NICK SOTNICHEK ''AS IS'' AND ANY
    EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
    WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
    DISCLAIMED. IN NO EVENT SHALL NICK SOTNICHEK BE LIABLE FOR ANY
    DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
    (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
    LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
    ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*/

// Country codes from ISO 3166-1 ftp://ftp.ripe.net/iso3166-countrycodes.txt

#ifndef COUNTRYSELECTOR_H
#define COUNTRYSELECTOR_H

#include <Plasma/Label>
#include <Plasma/Frame>
#include <Plasma/LineEdit>
#include <Plasma/ComboBox>
#include <Plasma/PushButton>

struct country
{
  QString name;
  QString a2;
  QString a3;
  int number;
  bool postal;
};
 
class CountrySelector : public QObject
{
	Q_OBJECT
	public:
		CountrySelector(QGraphicsWidget *parent);
		virtual ~CountrySelector();
		void setVisible(bool enabled);		
		void setCountry(QString ss, QString cs);
		void setGeomerty(QRect rect);		

		
		void paintInterface(QPainter *painter,
                const QStyleOptionGraphicsItem *option,
                const QRect& contentsRect);

	public slots:
		void Ok();
		void Cancel();
		void CountryChanged(const QString& text);
		void PostalChanged(const QString &text);
	  
	signals:
		void CountrySelected(QString,QString);
		void CountryCanceled();
		
	protected:
		void filllist();
		void initFrame();

	private:
		bool visible;
		QString s_country;
		QString s_postal;
		QGraphicsWidget *parent;
		Plasma::Frame *mFrame;
		Plasma::Label *mLabel1;
		Plasma::Label *mLabel2;
		Plasma::PushButton *mOk;
		Plasma::PushButton *mCancel;
		Plasma::ComboBox *mCBox;
		Plasma::LineEdit *mLEdit;
		QList<country> countrylist;		
};

#endif // COUNTRYSELECTOR_H
