/*
    Copyright (c) 2010, Nick Sotnichek
    All rights reserved.

    Redistribution and use in source and binary forms, with or without
    modification, are permitted provided that the following conditions are met:
        * Redistributions of source code must retain the above copyright
        notice, this list of conditions and the following disclaimer.
        * Redistributions in binary form must reproduce the above copyright
        notice, this list of conditions and the following disclaimer in the
        documentation and/or other materials provided with the distribution.
        * Neither the name of the author nor the
        names of its contributors may be used to endorse or promote products
        derived from this software without specific prior written permission.

    THIS SOFTWARE IS PROVIDED BY NICK SOTNICHEK ''AS IS'' AND ANY
    EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
    WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
    DISCLAIMED. IN NO EVENT SHALL NICK SOTNICHEK BE LIABLE FOR ANY
    DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
    (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
    LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
    ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*/

#include "plasmoid_for_ebay_item.h"
#include <QPainter>
#include <QFontMetrics> 
#include <QSizeF>
#include <KLocale>
#include <KNotification>
#include <QFile>
#include <QTimer>
#include <QGraphicsLinearLayout>
#include <QGraphicsGridLayout>
#include <QImage>
#include <QWidget>
#include <QDesktopServices>
#include <QtNetwork>
#include <QUrl>
#include <QNetworkRequest>
#include <QNetworkReply>
#include <QXmlStreamReader>
#include <QClipboard>
#include <QStringList>
#include <QApplication>
#include <QNetworkProxyFactory>
#include <QProcess>

#include <plasma/svg.h>
#include <plasma/theme.h>
#include <plasma/widgets/lineedit.h>
#include <plasma/widgets/pushbutton.h>
#include <plasma/widgets/frame.h>
#include <plasma/widgets/label.h>

#include <kprotocolmanager.h>

#include <syslog.h>

plasmoid_for_eBay_item::plasmoid_for_eBay_item(QObject *parent, const QVariantList &args)
    : Plasma::Applet(parent, args),
    m_svg(this)
 {
    
    KAboutData about(A_PROGRAM_NAME, 0, ki18n(A_PROGRAM_NAME), A_VERSION, ki18n("PLasma for ebay ITEM viewer."),
                     KAboutData::License_GPL, ki18n("(C) 2010 %{AUTHOR}"), KLocalizedString(), 0, "%{EMAIL}");
    about.addAuthor( ki18n("%{AUTHOR}"), KLocalizedString(), "%{EMAIL}" );
   
   
    setBackgroundHints(DefaultBackground);
    setHasConfigurationInterface(false);  
    setAspectRatioMode(Plasma::FixedSize);
    setAcceptDrops(true);
   
    resize(270, 132); 

    m_timer1 = new QTimer(this);
    m_timer2 = new QTimer(this);
    m_timer3 = new QTimer(this);
    m_timer4 = new QTimer(this); //timer for coonection timeout
    m_timer2->setInterval(1000);
    m_timer3->setInterval(1000); //11.06.11 changed from 10000
//    m_timer3->start();	// 11.06.11 timer start moved after config loading
    m_timer4->setInterval(A_CONNECT_TIMEOUT);

    connect(m_timer1, SIGNAL(timeout()), this, SLOT(urlReturnStatus()));
    connect(m_timer2, SIGNAL(timeout()), this, SLOT(timetick()));
    connect(m_timer3, SIGNAL(timeout()), this, SLOT(timertick3()));
    connect(m_timer4, SIGNAL(timeout()), this, SLOT(timertick4()));
    
    createNewConnectionManager();
    
    loadcountryandpostal();   
    clearoldconfig();
    proxySource=0;
   
    openlog("Plasmoid for ebay item", LOG_PID, LOG_USER);

}

plasmoid_for_eBay_item::~plasmoid_for_eBay_item()
{
    closelog();
    if (hasFailedToLaunch()) 
    {

    } 
    else 
    {
	  saveMyState();
	  if(notify_showes)
	  {
	    notify_showes=false;
	    notification->close();
	  }
    }
}

void plasmoid_for_eBay_item::createNewConnectionManager()
{
    m_d.manager = new QNetworkAccessManager( this );
    connect( m_d.manager, SIGNAL(finished(QNetworkReply*)),this, SLOT(finished(QNetworkReply *)) );  
    m_d.picmanager = new QNetworkAccessManager( this );
    connect( m_d.picmanager, SIGNAL(finished(QNetworkReply*)),this, SLOT(finishedpic(QNetworkReply *)) );    
}

void plasmoid_for_eBay_item::dropOldConnectionManager()
{
  m_d.manager->deleteLater();
  m_d.picmanager->deleteLater();
}

void plasmoid_for_eBay_item::init()
{
	
	ClearItem(); 
	setProxy();
	

	QString ff="Sans Serif";
	label = new Plasma::Label(this); //Название итема
	label->setVisible(false);	
        label->setGeometry(QRect(contentsRect().left() + 8,contentsRect().top() + 2, contentsRect().width() - 16, 20));
        QFont font;
        font.setPixelSize (7);
        font.setBold(true);
        font.setWeight(75);
	font.setFamily(ff);
        label->setFont(font);
        label->setLayoutDirection(Qt::LeftToRight);
        label->setAlignment(Qt::AlignCenter|Qt::AlignHCenter);
	label->setText("Item title here");

	is_item_fl=0;

	int yy=23;
	int xx=83;
	int dx=85;
	int dy=12;
	int w=84;
	int h=13;
	int fsize=11;
	fl_x=143;
	fl_y=57;
	s_x=148;
	s_y=yy+4*dy;

	configGroup="";
			
	QFont font9;
        font9.setPixelSize (fsize);
	font9.setFamily(ff);
	
        label_2 = new Plasma::Label(this);
	label_2 ->setVisible(false);
	label_2 ->setFont(font9);
        label_2->setGeometry(QRect(contentsRect().left() + xx,contentsRect().top() + yy, w, h));
        label_2->setAlignment(Qt::AlignRight|Qt::AlignTop|Qt::AlignTrailing);
	label_2->setText("Bid history :");

	
        label_3 = new Plasma::Label(this);
	label_3 ->setVisible(false);	
	label_3 ->setFont(font9);
        label_3->setGeometry(QRect(contentsRect().left() + xx,contentsRect().top() + yy+dy, w, h));
        label_3->setAlignment(Qt::AlignRight|Qt::AlignTop|Qt::AlignTrailing);
	label_3->setText("Time left :");

	
	
        label_4 = new Plasma::Label(this);
	label_4 ->setVisible(false);		
	label_4 ->setFont(font9);
        label_4->setGeometry(QRect(contentsRect().left() + xx,contentsRect().top() + yy+2*dy, w, h));
	label_4->setAlignment(Qt::AlignRight|Qt::AlignTop|Qt::AlignTrailing);
	label_4->setText("Current bid :");
	
        label_5 = new Plasma::Label(this);
	label_5 ->setVisible(false);			
	label_5 ->setFont(font9);
        label_5->setGeometry(QRect(contentsRect().left() + xx,contentsRect().top() + yy+3*dy, w, h));
        label_5->setAlignment(Qt::AlignRight|Qt::AlignTop|Qt::AlignTrailing);
	label_5->setText(":");

        label_5a = new Plasma::Label(this);
	label_5a ->setVisible(false);			
	label_5a ->setFont(font9);
        label_5a ->setGeometry(QRect(contentsRect().left() + fl_x-w,contentsRect().top() + yy+3*dy, w, h));
        label_5a ->setAlignment(Qt::AlignRight|Qt::AlignTop|Qt::AlignTrailing);
	label_5a ->setText("Shipping");	

        QFont font1;
        font1.setBold(true);
        font1.setWeight(75);
	font1.setPixelSize (fsize);
	font1.setFamily(ff);	
        label_6 = new Plasma::Label(this);
	label_6 ->setVisible(false);		
        label_6->setGeometry(QRect(contentsRect().left() + xx,contentsRect().top() + yy+4*dy+1, w, h));
        label_6->setFont(font1);
        label_6->setAlignment(Qt::AlignRight|Qt::AlignTop|Qt::AlignTrailing);
	label_6->setText(":");

        label_6a = new Plasma::Label(this);
	label_6a -> setVisible(false);		
        label_6a -> setGeometry(QRect(contentsRect().left() + s_x-1-w,contentsRect().top() + yy+4*dy+1, w, h));
        label_6a -> setFont(font1);
        label_6a -> setAlignment(Qt::AlignRight|Qt::AlignTop|Qt::AlignTrailing);
	label_6a -> setText("Current");	
	
        label_bids = new Plasma::Label(this);
	label_bids ->setVisible(false);			
	label_bids ->setFont(font9);
        label_bids->setGeometry(QRect(contentsRect().left() + xx+dx,contentsRect().top() + yy, w, h));
        label_bids->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);
	label_bids->setText("0");
	
        label_timeleft = new Plasma::Label(this);
	label_timeleft ->setVisible(false);		
	label_timeleft ->setFont(font9);
        label_timeleft->setGeometry(QRect(contentsRect().left() + xx+dx,contentsRect().top() + yy+dy, w, h));
        label_timeleft->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);
	label_timeleft->setText("00h 00m");

        label_timeleftb = new Plasma::Label(this);
	label_timeleftb ->setVisible(false);		
	label_timeleftb ->setFont(font9);
        label_timeleftb->setGeometry(QRect(contentsRect().left() + xx+dx,contentsRect().top() + yy+dy, w, h));
        label_timeleftb->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);
	label_timeleftb->setText("00h 00m");	
	
        label_currbid = new Plasma::Label(this);
	label_currbid ->setVisible(false);			
	label_currbid ->setFont(font9);
        label_currbid->setGeometry(QRect(contentsRect().left() + xx+dx,contentsRect().top() + yy+2*dy, w, h));
        label_currbid->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);
	label_currbid->setText("US $0");

        label_shipping = new Plasma::Label(this);
	label_shipping ->setVisible(false);			
	label_shipping ->setFont(font9);
        label_shipping->setGeometry(QRect(contentsRect().left() + xx+dx,contentsRect().top() + yy+3*dy, w, h));
        label_shipping->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);
	label_shipping->setText("Not specified");
	
	label_totalprice = new Plasma::Label(this);
	label_totalprice->setVisible(false);	
        label_totalprice->setGeometry(QRect(contentsRect().left() + xx+dx,contentsRect().top() + yy+4*dy+1, w, h));
        label_totalprice->setFont(font1);
        label_totalprice->setAlignment(Qt::AlignLeading|Qt::AlignLeft|Qt::AlignTop);
	label_totalprice->setText("US $0");
    
	applet_url = new UrlLabel(this,true);
	applet_url -> setGeomerty(QRect(contentsRect().right()-20,contentsRect().bottom()-20,20,20));
	applet_url -> setText(QString("w"));
	applet_url -> setImage(":/img/url.png");
	applet_url -> setEnabled(true);
	applet_url -> setVisible(true);
	applet_url -> setToolTip(tr("Visit plasmoid homepage."));
	connect(applet_url,SIGNAL(clicked()),this,SLOT(p_homepage()));

	applet_paste = new UrlLabel(this,true);
	applet_paste -> setGeomerty(QRect(contentsRect().right()-42,contentsRect().bottom()-20,20,20));
	applet_paste -> setText(QString("w"));
	applet_paste -> setImage(":/img/paste.png");
	applet_paste -> setEnabled(true);
	applet_paste -> setVisible(true);	
	applet_paste -> setToolTip("Paste item number or item url.");
	connect(applet_paste,SIGNAL(clicked()),this,SLOT(p_paste()));	

	applet_but = new UrlLabel(this,true);
	applet_but -> setGeomerty(QRect(contentsRect().left()+xx+dx-43,contentsRect().top()+yy+dy*5+1,86,22));
	applet_but -> setText(QString("Place bid"));
	applet_but -> setImage(":/img/Place_bid.png");
	applet_but -> setEnabled(true);
	applet_but-> setVisible(false);	
	applet_but -> setToolTip("");
	connect(applet_but,SIGNAL(clicked()),this,SLOT(PlaceBidClicked()));	

	applet_flag = new UrlLabel(this,true);
	applet_flag -> setGeomerty(QRect(contentsRect().left() + fl_x,contentsRect().top() + fl_y, 22, 16));
	applet_flag -> setText(QString("Change country."));
	applet_flag -> setEnabled(true);
	applet_flag-> setVisible(false);	
	applet_flag -> setToolTip("Change country.");

	
	
	cs_dialog = new CountrySelector(this);
	cs_dialog -> setGeomerty(QRect(contentsRect().left()+(contentsRect().width()-233)/2,contentsRect().top()+(contentsRect().height()-98)/2, 233, 98)); //contentsRect().height()));
	connect(applet_flag,SIGNAL(clicked()),this,SLOT(selectcountry()));
	connect(cs_dialog,SIGNAL(CountryCanceled()),this,SLOT(cancelcountry()));
	connect(cs_dialog,SIGNAL(CountrySelected(QString,QString)),this,SLOT(selectedcountry(QString,QString)));
	
	m_image=QImage(":/img/Right_Now_108x45.png");
	
	url_width=0;
	url_height=0;
	url_left=0;
	url_top=0;
	
//	state=0;
//	state=30;
//	state=22;
	state=-1;
	
	sprintf(m_text," ");
	item_num="";
	dcount=0;
	setBusy(true);
	m_timer1->start(1000);

//	notification= new KNotification ( "lesswhen5min", KNotification::Persistant);
      	notification= new KNotification ( "lesswhen5min");
	notification->setFlags(KNotification::Persistant);
	notify_showes=false;
	connect(notification, SIGNAL(activated(unsigned int )), this , SLOT(PlaceBidClickedFromNotify()) );
	connect(notification, SIGNAL(closed()), this , SLOT(notifyclosed()) );
}

bool plasmoid_for_eBay_item::setProxyFromString(QString s_proxy)
{
	    int p0=s_proxy.indexOf(QString("http://"));
	    if(p0>=0)
	    {
	      s_proxy=s_proxy.mid(p0+7);
	      p0=s_proxy.indexOf("/");
	      if(p0>0)
		s_proxy=s_proxy.left(p0);	      
	      QString s_hostname=s_proxy;
	      const QString s_default_port="80";
	      QString s_port=s_default_port, s_user=QString(), s_password=QString();
	      p0=s_proxy.indexOf(QString("@"));	      
	      if(p0>0)
	      {
		if(p0<s_proxy.length()-1)
		{
		  s_user=s_proxy.left(p0);				  
		  s_hostname=s_proxy.mid(p0+1);
		  if(s_user.length()>0)
		  {
		    p0=s_user.indexOf(":");
		    if(p0>0)
		    {
		      if(p0<s_user.length()-1)
			s_password=s_user.mid(p0+1);
		      s_user=s_user.left(p0);
		    }
		    else
		      s_user=QString();
		  }
		}
	      }
	      p0=s_hostname.indexOf(QString(":"));	      
	      if(p0>0)
	      {
		if(p0<s_hostname.length()-1)
		  s_port=s_hostname.mid(p0+1);
		s_hostname=s_hostname.left(p0);
		int i;
		for(i=0;i<s_port.length();i++)
		  if(!(s_port[i]==QChar('0')||s_port[i]==QChar('1')||s_port[i]==QChar('2')||s_port[i]==QChar('3')||s_port[i]==QChar('4')||s_port[i]==QChar('5')||s_port[i]==QChar('6')||s_port[i]==QChar('7')||s_port[i]==QChar('8')||s_port[i]==QChar('9')))
		    break;
		if(i>0)
		{
		  if(i<s_port.length())
		    s_port=s_port.left(i);
		}
		else
		  s_port=s_default_port;
	      }
	      if(s_hostname.length()>0)
	      {
		bool bok;
		uint uport=s_port.toUShort(&bok,10);
		if(!bok)
		  uport=80;
		
//		dstr = QString("h:")+s_hostname+QString("\np:")+QString(uport)+QString("\nl:")+ s_user + QString("\nl:")+ s_password;
		qproxy=QNetworkProxy(QNetworkProxy::HttpProxy, s_hostname, uport, s_user, s_password);
		m_d.manager->setProxy(qproxy);
		m_d.picmanager->setProxy(qproxy);
		return true;
	      }
	    }
	    return false;
}

void plasmoid_for_eBay_item::setProxy()
{
    QList<QNetworkProxy> proxyList;
    char *cp_proxy;
    QProcess process;    
    
    switch (proxySource)
    {
      case 0:
	//Установка параметров proxy из переменных среды	
	cp_proxy=getenv ("http_proxy");
	if(cp_proxy!=NULL)
	{
	  if(setProxyFromString(QString(cp_proxy)))
	  {
	      	proxySource++;
		break;
	  }
	}
	proxySource=2;
     // break не нужно так как есди формат пременной среды не правильный, то определять пркси другим способом
     case 2:
       //Установка параметров proxy из kreadconfig --file kioslaverc --group \"Proxy Settings\" --key httpProxy
       //now from KProtocolManager
       if(KProtocolManager::useProxy())
       {
	    QString  s_proxy=KProtocolManager::proxyForUrl(KUrl(QString(A_SERVICE_URL1)));
	    if(s_proxy.length() > 0 && s_proxy!="DIRECT")
	    {
	      if(setProxyFromString(s_proxy))
	      {
		  proxySource++;
		  break;
	      }
	    }
       }       
       proxySource=4;
      // break не нужно так как есди формат пременной среды не правильный, то определять пркси другим способом       
     case 4:
     //Установка параметров proxy из QNetworkProxyFactory::systemProxyForQuery
	proxyList=QNetworkProxyFactory::systemProxyForQuery (QNetworkProxyQuery(QUrl(QString(A_SERVICE_URL1))));
	qproxy=proxyList[0];
	m_d.manager->setProxy(qproxy);
	m_d.picmanager->setProxy(qproxy);     
	proxySource++;
     break;
     case 5:
     //Не использовать прокси
	qproxy.setType(QNetworkProxy::NoProxy);
	proxySource=0;
     break;          
     case 1:
     case 3:  
     //Параметры proxy из переменных среды установлены, переключить HostNameLookupCapability
	qproxy.setCapabilities(qproxy.capabilities()^QNetworkProxy::HostNameLookupCapability);
	proxySource++;
     break;
   }
/*   int cap=0;
   if(qproxy.capabilities().testFlag(QNetworkProxy::HostNameLookupCapability))
     cap=1;
   int cnt=proxyList.count();
   syslog(0, "proxySource=%d, proxyHostname=%s, proxyPort=%d, proxyHostnameLookupCapability=%d, proxyUsername=%s, proxyPassword=%s, count=%d, type=%d", proxySource, qproxy.hostName().toAscii().data(), qproxy.port(), cap, qproxy.user().toAscii().data(), qproxy.password().toAscii().data(), cnt, qproxy.type()  );*/
}

void plasmoid_for_eBay_item::selectcountry()
{
  cs_dialog->setCountry(m_d.s_country,m_d.s_postalcode);
  cs_dialog->setVisible(true);
  state=22;
  LabelsOFF();
  update();
}

void plasmoid_for_eBay_item::cancelcountry()
{
  cs_dialog->setVisible(false);
  state=30;
  LabelsON();
  update();  
}

void plasmoid_for_eBay_item::selectedcountry(QString cs, QString ps)
{
  cs_dialog->setVisible(false);
  m_d.s_country=cs;
  m_d.s_postalcode=ps;
  if(QFile::exists(QString(":/img/flag/")+cs+QString(".png")))  
    m_d.flag=QImage(QString(":/img/flag/")+cs+QString(".png"));
  else
    m_d.flag=QImage(QString(":/img/flag/unknown.png"));
  savecountryandpostal();
  dropOrPasteEvent();
}

void plasmoid_for_eBay_item::PlaceBidClicked()
{
    if(m_d.s_url.length()>0&&m_d.s_url.indexOf("http")==0)
    {
	if(m_d.s_orig_site.length()>0)
	{
	  int p0=m_d.s_url.mid(8).indexOf("/");
	  if(p0>0)
	  {
	      QString nurl="http://"+m_d.s_orig_site+m_d.s_url.mid(8+p0);
	      QDesktopServices::openUrl(QUrl::fromEncoded( nurl.toAscii() ));
	      return;
	  }
	}
	if(m_d.s_site.length()>0)
	{
	  int p0=m_d.s_url.mid(8).indexOf("/");
	  if(p0>0)
	  {
	      QString nurl="http://"+m_d.s_site+m_d.s_url.mid(8+p0);
	      QDesktopServices::openUrl(QUrl::fromEncoded( nurl.toAscii() ));
	      return;
	  }
	}	
	QDesktopServices::openUrl(QUrl::fromEncoded( m_d.s_url.toAscii() ));
    }
}

void plasmoid_for_eBay_item::p_homepage()
{
	QDesktopServices::openUrl(QUrl::fromEncoded( A_PROGRAM_URL ));
}

void plasmoid_for_eBay_item::timertick4() //timer for request timeout
{
     qnreply->abort();
     m_timer4->stop();
//     connectionFailed();
}


void plasmoid_for_eBay_item::connectionFailed()
{
	    sprintf(m_text,"Waiting for\ninternet connection.");
	    state = 12;
	    this->setBusy(false);
	    LabelsOFF();
	    dropOldConnectionManager();
	    createNewConnectionManager();
	    setProxy();
	    m_timer1->start(1000);		  
	    update();	    
	    return;  
}

 void plasmoid_for_eBay_item::finished( QNetworkReply *reply )
 {
   m_timer4->stop();
   if ( reply->error() != QNetworkReply::NoError ) 
    {      
	connectionFailed();
    }
    processPhpResult( reply );
   
 }

void plasmoid_for_eBay_item::finishedpic( QNetworkReply *reply )
 {
    update();
    if(state==20)
	return;
    if ( reply->error() == QNetworkReply::NoError ) 
    {
	if(!m_d.pic.loadFromData(reply->readAll()))
	  m_d.s_result += " Pic load readAll error.";
	else
	  m_d.s_result += " Pic load OK.";
    }
    else
    {	
	char s[200];
	sprintf(s,"code=%d. QNetworkReply::NoError=%d.",reply->error(), QNetworkReply::NoError);
        m_d.s_result += ". Pic load error:" + QString(s);
    }
    fl_notify=true;
    setstate_25();	
    return;
 }

void plasmoid_for_eBay_item::setstate_25()
{
  this->setBusy(false);
  if(m_d.s_isnewversion=="1"&&is_item_fl!=1)
  {
    state=25;
    is_item_fl=1;
    applet_url->setVisible(true);
    m_timer1->start(5000);	    
  }
  else
  {
    setstate_30();
    m_timer2->start();
  }
  update();
}
void plasmoid_for_eBay_item::setstate_30()
{
  
    state=30;    
    this->setBusy(false);

    label -> setText(m_d.s_title);
    label -> setVisible(true);			
    if(m_d.s_title.length()<28)
    {
	QFont font;
        font.setPixelSize (12);
	font.setBold(true);
        label->setFont(font); 
    }
    else
    {
	QFont font;
        font.setPixelSize (9);
	font.setBold(false);	
        label->setFont(font); 	
    }
    
    if(m_d.s_listingtype=="FixedPriceItem")
      label_bids -> setText(m_d.s_quantity);
    else
    {
      if(m_d.s_highbidder.length()>0&&m_d.s_bidcount!="0")
	label_bids -> setText(m_d.s_bidcount+" ("+m_d.s_highbidder+")");
      else
	label_bids -> setText(m_d.s_bidcount);
    }
    label_bids -> setVisible(true);			

    if(m_d.s_listingstatus=="Active")
    {
      label_timeleft->setText(m_d.s_sinterval);
      label_timeleftb->setText(m_d.s_sinterval);     
      applet_but->setEnabled(true);
      applet_but->setCursor(Qt::PointingHandCursor);
      if(m_d.s_cinterval=="1")
      {
	QFont font1;
	font1.setBold(true);
	font1.setPixelSize (11);
	label_timeleftb->setFont(font1);
	label_timeleftb->setStyleSheet("color: rgb(255, 0, 0);");
	label_timeleftb ->setVisible(true);
	label_timeleft ->setVisible(false);
      }
      else
      {
	QFont font1;
	font1.setBold(false);
	font1.setPixelSize (11);
	label_timeleft->setFont(font1);
	label_timeleft->setStyleSheet("color: rgb(0, 0, 0);");
	label_timeleft->setFont(font1);	
	label_timeleft ->setVisible(true);
	label_timeleftb ->setVisible(false);
      }    
    }
    else
    {
	QFont font1;
	font1.setBold(true);
	font1.setPixelSize (11);
	label_timeleftb->setFont(font1);
	label_timeleftb->setStyleSheet("color: rgb(0, 0, 255);");
	label_timeleftb ->setVisible(true);
	label_timeleft ->setVisible(false);      
	label_timeleft->setText(m_d.s_listingstatus);
	label_timeleftb->setText(m_d.s_listingstatus);          
	applet_but->setEnabled(false);
	applet_but->setCursor(Qt::ArrowCursor);
	m_timer2->stop();
    }
	
    label_currbid->setText(m_d.s_currentprice);
    label_currbid->setVisible(true);			
	
    label_shipping->setText(m_d.s_shprice);    
    label_shipping->setVisible(true);			
    
    label_totalprice->setText(m_d.s_totalprice);
    label_totalprice->setVisible(true);	
    label_2->setVisible(true);
    label_3->setVisible(true);    
    label_4->setVisible(true);    
    label_5->setVisible(true);
    label_5a->setVisible(true);    
    label_6->setVisible(true);
    label_6a->setVisible(true);    
    applet_url->setVisible(false);
    applet_paste->setVisible(false);
    applet_but->setVisible(true);
    applet_flag->setVisible(true);
    
    is_item_fl=1;
    
    update();	
  
  
}



bool plasmoid_for_eBay_item::processPhpResult( QIODevice *source )
{
    QXmlStreamReader reader( source );
    int fl=0;
    while ( !reader.atEnd() ) 
    {
        QXmlStreamReader::TokenType tokenType = reader.readNext();
        if ( tokenType == QXmlStreamReader::StartElement ) 
	{
            if ( reader.name() == QString("errcode") ) 
	    {
//		m_d.s_result = reader.readElementText(QXmlStreamReader::ErrorOnUnexpectedElement);
		m_d.s_result = reader.readElementText();
		if(m_d.s_result != "OK")
		{
		  sprintf(m_text,"Error: %s",(char *)m_d.s_result.left(200).toAscii().constData());
		  state = 11;
		  this->setBusy(false);
		  m_timer1->start(1000);		  
		  update();
		  return false;
		}
            }
            if ( reader.name() == QString("number") ) 
		m_d.s_itemnumber = reader.readElementText();	      
            if ( reader.name() == QString("ssite") ) 
		m_d.s_site = reader.readElementText();	      	    
//		m_d.s_itemnumber = reader.readElementText(QXmlStreamReader::ErrorOnUnexpectedElement);
            if ( reader.name() == QString("url") ) 
		m_d.s_url = reader.readElementText();	      
//		m_d.s_url = reader.readElementText(QXmlStreamReader::ErrorOnUnexpectedElement);
            if ( reader.name() == QString("picurl") ) 
	    {
		m_d.s_picurl = reader.readElementText();
//		m_d.s_picurl = reader.readElementText(QXmlStreamReader::ErrorOnUnexpectedElement);		
	        if(m_d.s_picurl.length()>0)
		    if(m_d.s_picurl.indexOf(QString("http://"),0,Qt::CaseInsensitive)==0)
		      fl=1;
	    }
            if ( reader.name() == QString("title") ) 
		m_d.s_title = reader.readElementText();
//		m_d.s_title = reader.readElementText(QXmlStreamReader::ErrorOnUnexpectedElement);	    
            if ( reader.name() == QString("bidcount") ) 
		m_d.s_bidcount = reader.readElementText();	    
            if ( reader.name() == QString("highbidder") ) 
		m_d.s_highbidder = reader.readElementText();	    
//		m_d.s_bidcount = reader.readElementText(QXmlStreamReader::ErrorOnUnexpectedElement);	    
            if ( reader.name() == QString("currprice") ) 
	    {
		m_d.s_currentprice = reader.readElementText();
//		m_d.s_currentprice = reader.readElementText(QXmlStreamReader::ErrorOnUnexpectedElement);		
		m_d.s_currentprice_d = m_d.s_currentprice;
	    }
            if ( reader.name() == QString("listingtype") ) 
		m_d.s_listingtype = reader.readElementText();
//		m_d.s_listingtype = reader.readElementText(QXmlStreamReader::ErrorOnUnexpectedElement);	    
            if ( reader.name() == QString("listingstatus") ) 
		m_d.s_listingstatus = reader.readElementText();
//		m_d.s_listingstatus = reader.readElementText(QXmlStreamReader::ErrorOnUnexpectedElement);	    
            if ( reader.name() == QString("condition") ) 
		m_d.s_condition = reader.readElementText();
//		m_d.s_condition = reader.readElementText(QXmlStreamReader::ErrorOnUnexpectedElement);	    
            if ( reader.name() == QString("shprice") ) 
	    {
//		m_d.s_shprice = reader.readElementText(QXmlStreamReader::ErrorOnUnexpectedElement);
		m_d.s_shprice = reader.readElementText();		
		m_d.s_shprice_d=m_d.s_shprice;
	    }
            if ( reader.name() == QString("shcurr") ) 
		m_d.s_shcurr = reader.readElementText();	    
//		m_d.s_shcurr = reader.readElementText(QXmlStreamReader::ErrorOnUnexpectedElement);	    	    
            if ( reader.name() == QString("lesswhenhour") ) 
		m_d.s_cinterval = reader.readElementText();	      
//		m_d.s_cinterval = reader.readElementText(QXmlStreamReader::ErrorOnUnexpectedElement);
            if ( reader.name() == QString("sinterval") ) 
		m_d.s_sinterval = reader.readElementText();
//		m_d.s_sinterval = reader.readElementText(QXmlStreamReader::ErrorOnUnexpectedElement);  	    
	    if ( reader.name() == QString("currency") ) 	
		m_d.s_currency = reader.readElementText();
//		m_d.s_currency = reader.readElementText(QXmlStreamReader::ErrorOnUnexpectedElement);  	    	    
	    if ( reader.name() == QString("rcurrency") ) 	
		m_d.s_rcurrency = reader.readElementText();
	    if ( reader.name() == QString("isnewversion") ) 
		m_d.s_isnewversion = reader.readElementText();
//		m_d.s_isnewversion = reader.readElementText(QXmlStreamReader::ErrorOnUnexpectedElement);  	    	    
	    if ( reader.name() == QString("daysleft") ) 
	    	m_d.s_daysleft = reader.readElementText();  	    
//	    	m_d.s_daysleft = reader.readElementText(QXmlStreamReader::ErrorOnUnexpectedElement);  	    	    
	    if ( reader.name() == QString("hoursleft") ) 
	    	m_d.s_hoursleft = reader.readElementText();
//	    	m_d.s_hoursleft = reader.readElementText(QXmlStreamReader::ErrorOnUnexpectedElement);  	    	    
	    if ( reader.name() == QString("minutesleft") ) 
	    	m_d.s_minutesleft = reader.readElementText();  	    
//	    	m_d.s_minutesleft = reader.readElementText(QXmlStreamReader::ErrorOnUnexpectedElement);  	    	    
	    if ( reader.name() == QString("secondsleft") ) 
	    	m_d.s_secondsleft = reader.readElementText();  	    
//	    	m_d.s_secondsleft = reader.readElementText(QXmlStreamReader::ErrorOnUnexpectedElement);  	    	    
	    if ( reader.name() == QString("quantity") ) 
	    	m_d.s_quantity = reader.readElementText();  	    	    
//	    	m_d.s_quantity = reader.readElementText(QXmlStreamReader::ErrorOnUnexpectedElement);  	    	    	    
	    if ( reader.name() == QString("country") ) 
	    {
//	    	m_d.s_country = reader.readElementText(QXmlStreamReader::ErrorOnUnexpectedElement);  	    	    	    
	    	m_d.s_country = reader.readElementText();  	    	    	    		
		QString s=QString(":/img/flag/")+m_d.s_country+QString(".png"); 
		m_d.flag=QImage(s); 
	    }
        }
        else if ( tokenType == QXmlStreamReader::Invalid )
	{
	    sprintf(m_text,"Error:\nWeb service error");
	    state = 12;
	    this->setBusy(false);
	    m_timer1->start(1000);		  
	    update();
	    return false;
	}
    }

    bool r2,r1;
    m_d.s_currentprice=m_d.s_currentprice_d;
    m_d.s_shprice=m_d.s_shprice_d;
    float shp=m_d.s_shprice.toFloat(&r1);
    float pr=m_d.s_currentprice.toFloat(&r2);
    if(r1&&r2&&m_d.s_shcurr==m_d.s_currency)  
    {
	m_d.s_totalprice=QString::number(shp+pr,'f',2);      
	m_d.s_totalprice=m_d.s_currency+m_d.s_totalprice;
    }
    else
      m_d.s_totalprice="n/a";
    m_d.s_currentprice=m_d.s_currency+QString::number(pr,'f',2);
    if(r1)
    {
      if(shp!=0)
	m_d.s_shprice=m_d.s_shcurr + QString::number(shp,'f',2);
      else
	m_d.s_shprice=tr("Free");
    }
    if(m_d.s_listingtype=="FixedPriceItem")
    {
      applet_but->setImage(":/img/Buy_it_now.png");
      label_2->setText("Available :");
    }
    else
    {
      applet_but->setImage(":/img/Place_bid.png");
      label_2->setText("Bid history :");
    }

    if(notify_showes)
      changenotify();

    if(fl==1)
	m_d.picmanager->get( QNetworkRequest(QUrl(m_d.s_picurl)));
    else
    {

      setstate_25();
    }
    
    saveMyState();
    return true;
}

 void plasmoid_for_eBay_item::dragEnterEvent(QGraphicsSceneDragDropEvent *event)
 {
     if (event->mimeData()->hasFormat("text/plain"))
     {
          event->acceptProposedAction();
	  sprintf(m_text,"Just drop it!");
	  
	  LabelsOFF();
	  
	  state=11;
	  applet_url->setVisible(false);
	  applet_paste->setVisible(false);
	  update();
     }
 }

void plasmoid_for_eBay_item::dragLeaveEvent(QGraphicsSceneDragDropEvent *event)
 {
     if (event->mimeData()->hasFormat("text/plain"))
     {
          event->acceptProposedAction();
	  sprintf(m_text," ");
	  
	  if(is_item_fl==1)
	  {  
	      LabelsON();
	      state=30;
	  }
	  else
	  {
	      state=0;
	      applet_url->setVisible(true);
	      applet_paste->setVisible(true);
	  }
	  update();
     }
 }
 
void plasmoid_for_eBay_item::keyPressEvent(QKeyEvent* event)
{
     if(event->key()==Qt::Key_F5)
     {
      if(state==30)
      {
	  m_d.apiUrl = QUrl(A_SERVICE_URL2);
	  m_d.apiUrl.addQueryItem("itemnum",m_d.s_itemnumber);
	  m_d.apiUrl.addQueryItem("version",QString(A_VERSION));
	  if(m_d.s_rcurrency!="")
	    m_d.apiUrl.addQueryItem("ucurrency",m_d.s_rcurrency);	  
	  qnreply = m_d.manager->get( QNetworkRequest(m_d.apiUrl));
	  m_timer4->start();
      }       
     }
     QGraphicsItem::keyPressEvent(event);         
}

 
  void plasmoid_for_eBay_item::LabelsOFF()
  {
	  label -> setVisible(false);			
	  label_bids -> setVisible(false);			
	  label_timeleft ->setVisible(false);		
	  label_timeleftb ->setVisible(false);		
	  label_currbid->setVisible(false);			
	  label_shipping->setVisible(false);			
	  label_totalprice->setVisible(false);	
	  label_2->setVisible(false);
	  label_3->setVisible(false);    
	  label_4->setVisible(false);    
	  label_5->setVisible(false);
	  label_5a->setVisible(false);	  
	  label_6->setVisible(false);  
	  label_6a->setVisible(false);  	  
	  applet_but->setVisible(false);
	  applet_flag->setVisible(false);
  }
  void plasmoid_for_eBay_item::LabelsON()
  {
	      label -> setVisible(true);			
	      label_bids -> setVisible(true);
	      if(m_d.s_cinterval=="1")
		label_timeleftb->setVisible(true);
	      else
		label_timeleft ->setVisible(true);		
	      label_currbid->setVisible(true);			
	      label_shipping->setVisible(true);			
	      label_totalprice->setVisible(true);	
	      label_2->setVisible(true);
	      label_3->setVisible(true);    
	      label_4->setVisible(true);    
	      label_5->setVisible(true);
	      label_5a->setVisible(true);	      
	      label_6->setVisible(true); 
	      label_6a->setVisible(true); 	      
	      applet_but->setVisible(true);
	      applet_flag->setVisible(true);
  }
 
 void plasmoid_for_eBay_item::dropEvent(QGraphicsSceneDragDropEvent *event)
 {
     if (event->mimeData()->hasFormat("text/plain"))
     {
	  item_num=event->mimeData()->text();
	  event->acceptProposedAction();     
	  dropOrPasteEvent();
     }
 }
  
 void plasmoid_for_eBay_item::dropOrPasteEvent()
 {
	  this->setBusy(true);	  
	  state=10;
          update();	  
	  int fl=0;
	  QString s_orig_site="";
	  QChar *data = item_num.data();	  
	  while (!data->isNull())  
	  {
	      if(!data->isDigit())
	      {
		  fl=1;
		  break;
	      }
	      ++data;
	  }
	  if(fl==1)
	  {
	      if(item_num.indexOf(QString("http://"),0,Qt::CaseInsensitive)>=0) //Выборка имени хоста сайта
	      {
		  int p0=item_num.indexOf(QString("http://"),0,Qt::CaseInsensitive);
		  s_orig_site=item_num.mid(p0+7);
		  p0=s_orig_site.indexOf(QString("/"),0,Qt::CaseInsensitive);
		  if(p0>0)
		    s_orig_site=s_orig_site.left(p0);
	      }
	      if(item_num.indexOf(QString("http://"),0,Qt::CaseInsensitive)!=0||(item_num.indexOf(QString("item"),0,Qt::CaseInsensitive)==-1&&item_num.indexOf(QString("-/"),0,Qt::CaseInsensitive)==-1))
		fl=2;
	      else
	      {
		     fl=2;
		     int pos=item_num.indexOf(QString("QQitemZ"));
		     if(pos>0)		       
		     {
			data+=pos+7;
			QString m_str;
		        while (!data->isNull()) 
			{
			  if(!data->isDigit())
			    break;
			  m_str.append(*data);			  
			  ++data;
			}
			if(m_str.length()>0)
			{
			    fl=1;
			    item_num=m_str;
			}
		     }
		     pos=item_num.indexOf(QString("hash=item"));
		     int di=9;
		     int base=16;
		     if(pos<0)
		     {
		       pos=item_num.indexOf(QString("ViewItem&item="),0,Qt::CaseInsensitive);
		       di=14;
		       base=10;
		     }
		     if(pos<0)
		     {
		       pos=item_num.indexOf(QString("&item="));
		       di=6;
		       base=10;		       
		     }
		     if(pos<0)
		     {
		       pos=item_num.indexOf(QString("-/"));
		       di=2;
		       base=10;		       
		     }		     
		     if(pos>0)		       
		     {
			data+=pos+di;
			QString m_str;
		        while (!data->isNull()&&((base==10&&data->isDigit())||(base==16&&(data->isDigit()||*data=='a'||*data=='b'||*data=='c'||*data=='d'||*data=='e'||*data=='f')))) 
			{
			  m_str.append(*data);			  
			  ++data;
			}
			if(m_str.length()>0)
			{
			    fl=1;
			    bool ok;
			    quint64 h = m_str.toULongLong(&ok, base);
			    if(ok)
			    {
				fl=1;
				item_num=item_num.setNum(h,10);
			    }
			    else
			    {
			      fl=2;
			    }
			}
		     }		     
	      }
	  }
	  if(fl==1||fl==0)
	  {
	      char s[255];
	      for(int i=0;i<200&&i<item_num.length();i++)
	      {
		  s[i]=item_num[i].toAscii();
		  s[i+1]='\0';
	      }
	      sprintf(m_text,"Item number:\n%s",s);
	  }
	  else
	  {
	      item_num="";	  
	      sprintf(m_text,"No item number.\n Try again.");
	      this->setBusy(false);
	      m_timer1->start(1000);
	  }
	  if(item_num.length()>0)
	  {	         
		 ClearItem(); 
		 if(s_orig_site.length()>0)
		   m_d.s_orig_site=s_orig_site;
		 m_d.apiUrl = QUrl(A_SERVICE_URL1);
		 m_d.apiUrl.addQueryItem("itemnum",item_num);
		 m_d.apiUrl.addQueryItem("version",QString(A_VERSION));
		 if(m_d.s_country.length()==2)
		   m_d.apiUrl.addQueryItem("country",m_d.s_country);
		 if(m_d.s_postalcode.length()>4)
		   m_d.apiUrl.addQueryItem("postalcode",m_d.s_postalcode);
		 qnreply = m_d.manager->get( QNetworkRequest(m_d.apiUrl));
		 m_timer4->start();
		 if(notify_showes)
		 {
		    notify_showes=false;
		    notification->close();
//		    notification= new KNotification ( "lesswhen5min", KNotification::Persistant);
		    notification= new KNotification ( "lesswhen5min");
		    notification->setFlags(KNotification::Persistant);
		    connect(notification, SIGNAL(activated(unsigned int )), this , SLOT(PlaceBidClickedFromNotify()) );
		    connect(notification, SIGNAL(closed()), this , SLOT(notifyclosed()) );		    
		 }
	  }
	  else
	  {
	      item_num="";	  
	      sprintf(m_text,"No item number.\n Try again.");
	      this->setBusy(false);
	      m_timer1->start(1000);	    
	  }
	  state=11;
	  applet_url->setVisible(false);
	  applet_paste->setVisible(false);
	  update();	  
 }
 

void plasmoid_for_eBay_item::clearoldconfig()
{
  KConfig config(A_PROGRAM_CONFIG);
  QStringList groups=config.groupList();
  for(int i=0;i<groups.size();i++)
  {
      if(groups[i].data()[0]!='X')
	continue;
      KConfigGroup group = config.group( groups[i] );
      QString i_num=group.readEntry("ItemID", QString(""));
      QDateTime dt=group.readEntry("DateTime",QDateTime::currentDateTime());
      QDateTime dt1=QDateTime::currentDateTime();
      dt=dt.addDays(30); 
      if(dt<dt1||i_num.length()<1)
      {
	  config.deleteGroup(groups[i]);
      }
  }
  config.sync();
}

 
void plasmoid_for_eBay_item::loadcountryandpostal()
{
  KConfig config(A_PROGRAM_CONFIG);
  KConfigGroup generalGroup( &config, "General" );
  m_d.s_country = generalGroup.readEntry("Country",QString(""));
  m_d.s_postalcode = generalGroup.readEntry("PostalCode",QString(""));
}

void plasmoid_for_eBay_item::savecountryandpostal()
{
  KConfig config(A_PROGRAM_CONFIG);
  KConfigGroup generalGroup( &config, "General" );
  generalGroup.writeEntry("Country",m_d.s_country);
  generalGroup.writeEntry("PostalCode",m_d.s_postalcode);    
  config.sync();
}

 
 void plasmoid_for_eBay_item::ClearItem()
 {
		 m_d.s_result = "";
		 m_d.s_itemnumber = "";
		 m_d.s_url = "";
		 m_d.s_picurl = "";
		 m_d.s_title = "";
		 m_d.s_listingtype = "";
		 m_d.s_currentprice = "";
		 m_d.s_currentprice_d="0";
		 m_d.s_bidcount = "";
		 m_d.s_listingstatus = "";
		 m_d.s_condition = "";
		 m_d.s_shprice = "n/a";
		 m_d.s_shprice_d = "-1";
		 m_d.s_shcurr = "";
		 m_d.s_cinterval = "";
		 m_d.s_sinterval = "";
		 m_d.pic=QImage(":/img/No_Image.png");
		 m_d.flag=QImage(":/img/flag/unknown.png");
		 m_d.sigma=QImage(":/img/sigma.png");
		 m_d.s_totalprice = "";
		 m_d.s_currency ="";
		 m_d.s_rcurrency ="";
		 m_d.s_isnewversion="0";
		 m_d.s_daysleft="0";
		 m_d.s_hoursleft="0";
		 m_d.s_minutesleft="0";
		 m_d.s_secondsleft="0";
		 m_d.s_quantity="";
		 m_d.s_highbidder="";
		 m_d.s_orig_site="";
		 m_d.s_site="";
		 fl_notify=false;
		 loadcountryandpostal();
 }

  void plasmoid_for_eBay_item::p_paste()
  {
     const QClipboard *clipboard = QApplication::clipboard();
     const QMimeData *mimeData = clipboard->mimeData(QClipboard::Clipboard);
     const QMimeData *mimeData1 = clipboard->mimeData(QClipboard::Selection);     
     if (mimeData1->hasText())
     {
	QString qs=mimeData1->text();
	if(qs.length()>8)
	{
		  item_num=qs;
		  LabelsOFF();
		  dropOrPasteEvent();
	}
     }     
     else
     {
       if(mimeData->hasText())
       {
	  QString qs=mimeData->text();
	  if(qs.length()>8)
	  {
		  item_num=qs;
		  LabelsOFF();
		  dropOrPasteEvent();
	  }	 
       }
     }
     update();
 }

void plasmoid_for_eBay_item::paintInterface(QPainter *p, const QStyleOptionGraphicsItem *option, const QRect &contentsRect)
{
 char message[1024];   
 QString sss;
 QImage im;

 switch (state)
 {
// 0 - начальное состояние
// 1 - мышь на url
// 2 - нажали url
  case 0:
//  case 1:
//  case 2:    
    p->setRenderHint(QPainter::SmoothPixmapTransform);
    p->setRenderHint(QPainter::Antialiasing);

    m_svg.resize((int)contentsRect.width(), (int)contentsRect.height());
    m_svg.paint(p, (int)contentsRect.left(), (int)contentsRect.top());

    p->drawImage((int)contentsRect.left(),(int)contentsRect.top(),m_image);
    p->save();
     
    sprintf(message,"\nDrag and drop\neBay item number\nor item page\nurl here.");
    
/*    message[0]='\0';
    for(int i=0;i<1023&&i<dstr.length();i++)
    {
      message[i]=dstr[i].toAscii();
      message[i+1]='\0';
    }*/
     
    p->setPen(QColor(0,0,153));
    m_font=QFont("Sans Serif",-1,-1,true);
    m_font.setBold(true);
    m_font.setStretch(100);
     
    p->setFont(m_font);     
     
    p->drawText(contentsRect,
                Qt::AlignCenter | Qt::AlignHCenter,
                message);
     m_font=QFont("Helvetica [Cronyx]");
     m_font.setItalic(false);
     m_font.setUnderline(false);
     m_font.setStretch(100);
     p->setFont(m_font);
     

     p->restore();
  break;
// Дропнули URL или Item_number.  
  case 10:
     p->setRenderHint(QPainter::SmoothPixmapTransform);
     p->setRenderHint(QPainter::Antialiasing);

     m_svg.resize((int)contentsRect.width(), (int)contentsRect.height());
     m_svg.paint(p, (int)contentsRect.left(), (int)contentsRect.top());

     p->drawImage((int)contentsRect.left(),(int)contentsRect.top(),m_image);
     p->save();
    
     sprintf(message,"\nReceiving data...");

     p->setPen(QColor(0,0,153));
     m_font=QFont("Sans Serif",-1,-1,true);
     m_font.setBold(true);
     m_font.setStretch(100);
     
     p->setFont(m_font);     
     
     p->drawText(contentsRect,
                Qt::AlignCenter | Qt::AlignBottom,
                message);


     p->restore();

  break;
// Определили item number или ошибка.  
  case 11:    
     p->setRenderHint(QPainter::SmoothPixmapTransform);
     p->setRenderHint(QPainter::Antialiasing);

     m_svg.resize((int)contentsRect.width(), (int)contentsRect.height());
     m_svg.paint(p, (int)contentsRect.left(), (int)contentsRect.top());

     p->drawImage((int)contentsRect.left(),(int)contentsRect.top(),m_image);
     p->save();

     sprintf(message,"%s",m_text);
     
     p->setPen(QColor(0,0,153));
     
     m_font=QFont("Sans Serif",-1,-1,true);
     m_font.setBold(true);
     m_font.setStretch(100);
     
     p->setFont(m_font);     
     
     p->drawText(contentsRect,
                Qt::AlignCenter | Qt::AlignBottom,
                message);


     p->restore();
     break;
// Ошибка сети
  case 12:    
     p->setRenderHint(QPainter::SmoothPixmapTransform);
     p->setRenderHint(QPainter::Antialiasing);

     m_svg.resize((int)contentsRect.width(), (int)contentsRect.height());
     m_svg.paint(p, (int)contentsRect.left(), (int)contentsRect.top());

     p->drawImage((int)contentsRect.left(),(int)contentsRect.top(),m_image);
     p->save();
     
     sprintf(message,"%s",m_text);

     p->setPen(QColor(0,0,153));
     
     m_font=QFont("Sans Serif",-1,-1,true);
     m_font.setBold(true);
     m_font.setStretch(100);
     
     p->setFont(m_font);     
     
     p->drawText(contentsRect,
                Qt::AlignCenter | Qt::AlignBottom,
                message);

     p->restore();     
  break;
  case 20:    
    p->setRenderHint(QPainter::SmoothPixmapTransform);
    p->setRenderHint(QPainter::Antialiasing);

    m_svg.resize((int)contentsRect.width(), (int)contentsRect.height());
    m_svg.paint(p, (int)contentsRect.left(), (int)contentsRect.top());
    
    p->drawImage((int)contentsRect.left(),(int)contentsRect.top(),m_d.pic);
    p->save();
    p->restore();
  break;
  
  case 22:    
    p->setRenderHint(QPainter::SmoothPixmapTransform);
    p->setRenderHint(QPainter::Antialiasing);

    m_svg.resize((int)contentsRect.width(), (int)contentsRect.height());
    m_svg.paint(p, (int)contentsRect.left(), (int)contentsRect.top());
    
    p->save();
    p->restore();
  break;  
  
  //Выдать сообщение о обновлении плазмоида
  case 25:    
     p->setRenderHint(QPainter::SmoothPixmapTransform);
     p->setRenderHint(QPainter::Antialiasing);

     m_svg.resize((int)contentsRect.width(), (int)contentsRect.height());
     m_svg.paint(p, (int)contentsRect.left(), (int)contentsRect.top());

     p->drawImage((int)contentsRect.left(),(int)contentsRect.top(),m_image);
     p->save();

     sprintf(message,"There is new\nversion of plasmoid.\nVisit plasmoid homepage ->");

     p->setPen(QColor(0,0,153));
     
     m_font=QFont("Sans Serif",-1,-1,true);
     m_font.setBold(true);
     m_font.setStretch(110);
     
     p->setFont(m_font);     
     
     p->drawText(contentsRect,
                Qt::AlignCenter | Qt::AlignBottom,
                message);


     p->restore();  
  break;
  case 30:    
    p->setRenderHint(QPainter::SmoothPixmapTransform);
    p->setRenderHint(QPainter::Antialiasing);

    m_svg.resize((int)contentsRect.width(), (int)contentsRect.height());
    m_svg.paint(p, (int)contentsRect.left(), (int)contentsRect.top());
    
    im=m_d.pic;
    if(m_d.pic.width()>80)
      im=m_d.pic.scaledToWidth(80);
    if(im.height()>80)
      im=im.scaledToHeight(80);
    p->drawImage((int)contentsRect.left()+10+(80-im.width())/2,(int)contentsRect.top() + 25 + (80-im.height())/2,im);
    p->drawImage((int)contentsRect.left()+fl_x+2,(int)contentsRect.top() + fl_y+2,m_d.flag);
    p->drawImage((int)contentsRect.left()+s_x,(int)contentsRect.top() + s_y+2,m_d.sigma);
    p->save(); 
    p->restore();
    break;
  default:
    ;
    
 }
 

}

void plasmoid_for_eBay_item::updateItemStatus()
{
  update();
}

void plasmoid_for_eBay_item::timertick3()
{
    	  xxx=this->pos().x();
	  yyy=this->pos().y();
	  QString curGroup=QString("X")+QString::number(xxx)+QString("Y")+QString::number(yyy)+QString("N");
	  if(configGroup=="")
	      configGroup=curGroup;
	  else
	  {
	    if(curGroup!=configGroup)
	      moveState();
	  }
}

void plasmoid_for_eBay_item::saveMyState()
{
    	  xxx=this->pos().x();
	  yyy=this->pos().y();
	  QString sgroup=QString("X")+QString::number(xxx)+QString("Y")+QString::number(yyy)+QString("N");
	  KConfig config (A_PROGRAM_CONFIG);
	  KConfigGroup group = config.group( sgroup );
	  group.writeEntry("ItemID", m_d.s_itemnumber);
	  if(m_d.s_orig_site.length()>0)
	    group.writeEntry("Site", m_d.s_orig_site);
	  group.writeEntry("ItemID", m_d.s_itemnumber);
	  QDateTime dt=QDateTime::currentDateTime();
	  group.writeEntry("DateTime",dt);
/*	  
	  int px = this->parentItem()->pos().x();
	  int py = this->parentItem()->pos().y();
	  group.writeEntry("parentx",px);
	  group.writeEntry("parenty",py);

	  int xx = this->pos().x();
	  int yy = this->pos().y();
	  group.writeEntry("x",xx);
	  group.writeEntry("y",yy);
*/	  	  
	  config.sync(); 
	  configGroup=sgroup;	  
}

QString plasmoid_for_eBay_item::moveState()
{
	  if(configGroup!="")
	      removeConfig(configGroup);
	  saveMyState();
	  return configGroup;
}

void plasmoid_for_eBay_item::removeConfig(QString configgrp)
{
  KConfig config(A_PROGRAM_CONFIG);
  QStringList groups=config.groupList();
  for(int i=0;i<groups.size();i++)
  {
      if(groups[i].data()[0]!='X')
	continue;
      if(groups[i]==configgrp)
      {
	  config.deleteGroup(groups[i]);
	  break;
      }
  }
  config.sync();	  
}

void plasmoid_for_eBay_item::urlReturnStatus()
{
    // Update status by timer
  	
    if(state==-1)
    {
	  KConfig config (A_PROGRAM_CONFIG);
	  setBusy(false);
	  
	  //for new configuration format X00Y00N
	  xxx=this->pos().x(); 
	  yyy=this->pos().y();
	  QString sgroup=QString("X")+QString::number(xxx)+QString("Y")+QString::number(yyy)+QString("N");
	  KConfigGroup group = config.group( sgroup );
	  QString i_num=group.readEntry("ItemID", QString(""));
	  QString s_orig_site=group.readEntry("Site", QString(""));
	  QDateTime dt=group.readEntry("DateTime",QDateTime::currentDateTime());
	  QDateTime dt1=QDateTime::currentDateTime();
	  dt=dt.addDays(30); 
	  m_timer3->start(); // 11.06.11 moved here to start moving sens after load config
	  if(dt>dt1)
	  {
	    item_num=m_d.s_itemnumber=i_num;
	    QChar *data = item_num.data();
	    int fl=0;
	    while (!data->isNull()) 
	    {
	      if(!data->isDigit())
	      {
		  fl=1;
		  break;
	      }
	      ++data;
	    }
	    if(fl==0&&item_num.length()>5)
	    {
	      dropOrPasteEvent();
	      m_d.s_orig_site=s_orig_site;	      
	      m_timer1->stop();
	      return;
	    }
	  }	  
	  
	  xxx=this->screenRect().left(); //for compatibiliti with old configuration format X00Y00
	  yyy=this->screenRect().top();
	  sgroup=QString("X")+QString::number(xxx)+QString("Y")+QString::number(yyy);
	  group = config.group( sgroup );
	  i_num=group.readEntry("ItemID", QString(""));
	  s_orig_site=group.readEntry("Site", QString(""));
	  dt=group.readEntry("DateTime",QDateTime::currentDateTime());
	  dt1=QDateTime::currentDateTime();
	  dt=dt.addDays(30); 
	  m_timer3->start(); // 11.06.11 moved here to start moving sens after load config
	  if(dt>dt1)
	  {
	    item_num=m_d.s_itemnumber=i_num;	    
	    QChar *data = item_num.data();
	    int fl=0;
	    while (!data->isNull()) 
	    {
	      if(!data->isDigit())
	      {
		  fl=1;
		  break;
	      }
	      ++data;
	    }
	    if(fl==0&&item_num.length()>5)
	    {
	      removeConfig(sgroup);
	      dropOrPasteEvent();
	      m_d.s_orig_site=s_orig_site;	      
	      m_timer1->stop();
	      return;
	    }
	  }
      }
      if(state==12)
      {
	 m_timer1->stop();
	 dropOrPasteEvent();
	 update();
	 return;
      }
      if(is_item_fl==0)
      {
	  state=0;
	  LabelsOFF();
	  applet_paste->setVisible(true);
	  applet_url->setVisible(true);
      }
      else
      {
	  if(state==25)
	    m_timer2->start();  
	  state=30;
	  setstate_30();
      }
  m_timer1->stop();
  update();
}

void plasmoid_for_eBay_item::timetick()
{
    int d=m_d.s_daysleft.toInt();
    int h=m_d.s_hoursleft.toInt();
    int i=m_d.s_minutesleft.toInt();
    int s=m_d.s_secondsleft.toInt();
    s--;
    if(s<0)
    {
      s=59;
      i--;
    }
    if(i<0)
    {
      i=59;
      h--;
    }
    if(h<0)
    {
      h=23;
      d--;	
    }
    if(d<0)
      d=h=i=s=0;
    QString ss;
    if(d>4)
	ss=QString::number(d)+QString("d");    
    else if(d>0)
	ss=QString::number(d)+QString("d ")+QString::number(h)+QString("h");
    else if(h>4)
        ss=QString::number(h)+QString("h");
    else if(h>0)
        ss=QString::number(h)+QString("h ")+QString::number(i)+QString("m");
    else if(i>4)
        ss=QString::number(i)+QString("m");
    else if(i>0)
        ss=QString::number(i)+QString("m ")+QString::number(s)+QString("s");
    else if(s>0)
        ss=QString::number(s)+QString("s");
    m_d.s_daysleft=QString::number(d);
    m_d.s_hoursleft=QString::number(h);
    m_d.s_minutesleft=QString::number(i);
    m_d.s_secondsleft=QString::number(s);
    if(d==0&&h==0)
      m_d.s_cinterval="1";
    else
      m_d.s_cinterval="0";
    if(m_d.s_listingstatus=="Active")
    {
      if(state==30)
      {
//	if((d>0&&h==0&&i==0&&s==0)||(d==0&&h>0&&i%10==0&&s==0)||(d==0&&h==0&&i>4&&i%5==0&&s==0)||(d==0&&h==0&&i<5&&s%20==0)||(d==0&&h==0&&i==0&&s>9&&s%10==0)||(d==0&&h==0&&i==0&&s<10&&s%3==0)||(d==0&&h==0&&i==0&&s==0))
	if((d>0&&i==0&&s==0)||(d==0&&h>0&&i%10==0&&s==0)||(d==0&&h==0&&i>4&&i%5==0&&s==0)||(d==0&&h==0&&i<5&&s%20==0)||(d==0&&h==0&&i==0&&s>9&&s%10==0)||(d==0&&h==0&&i==0&&s<10&&s%3==0)||(d==0&&h==0&&i==0&&s==0))	  
	{
	  m_d.apiUrl = QUrl(A_SERVICE_URL2);
	  m_d.apiUrl.addQueryItem("itemnum",m_d.s_itemnumber);
	  m_d.apiUrl.addQueryItem("version",QString(A_VERSION));
	  if(m_d.s_rcurrency!="")
	    m_d.apiUrl.addQueryItem("ucurrency",m_d.s_rcurrency);	  
	  qnreply = m_d.manager->get( QNetworkRequest(m_d.apiUrl));
	  m_timer4->start();
	}
	if(fl_notify&&d==0&&h==0&&i==4&&s==55)
	{
	    changenotify();
	    notification->sendEvent();
	    notify_showes=true;
	    fl_notify=false;
	}
      }
      if(ss!=m_d.s_currentprice)      
	m_d.s_sinterval=ss;
      if(state==30)
	setstate_30();
    }
    else
    {
      	  if(notify_showes)
	  {
	    notify_showes=false;
	    notification->close();
//	    notification= new KNotification ( "lesswhen5min", KNotification::Persistant);
	    notification= new KNotification ( "lesswhen5min");
	    notification->setFlags(KNotification::Persistant);
	    connect(notification, SIGNAL(activated(unsigned int )), this , SLOT(PlaceBidClickedFromNotify()) );
	    connect(notification, SIGNAL(closed()), this , SLOT(notifyclosed()) );
	  }
    }
//    xxx=this->screenRect().left(); changed becouse values updated by timetick3 timer every 1 second
//    yyy=this->screenRect().top();  
}

void plasmoid_for_eBay_item::changenotify()
{

  	    QString str=QString("<center><b>")+m_d.s_title+QString("</b></center>")+QString("<br>");
	    if(m_d.s_totalprice!="n/a")
	      str+=QString("<b>Total: ")+m_d.s_totalprice+QString("</b>");
	    else
	      str+=QString("<b>Current bid: ")+m_d.s_currentprice+QString("</b>");
	    notification->setText(str);
	    notification->setPixmap( QPixmap::fromImage(m_d.pic) );
	    QString ss=m_d.s_minutesleft;
	    ss=ss.setNum(ss.toInt()+1);
	    if(ss=="1")
		notification->setTitle(QString(" Less then ") + ss + QString(" minute left."));
	    else
		notification->setTitle(QString(" Less then ") + ss + QString(" minutes left."));
	    notification->setComponentData(KComponentData("plasmoid_for_ebay_item"));
	    if(m_d.s_listingtype=="FixedPriceItem")
	      notification->setActions( QStringList( i18n( "Buy it now" ) ) );
	    else
	      notification->setActions( QStringList( i18n( "Place bid" ) ) );	    
  
}

void plasmoid_for_eBay_item::notifyclosed()
{
    notify_showes=false;
//    notification= new KNotification ( "lesswhen5min", KNotification::Persistant);
    notification= new KNotification ( "lesswhen5min");
    notification->setFlags(KNotification::Persistant);
    connect(notification, SIGNAL(activated(unsigned int )), this , SLOT(PlaceBidClickedFromNotify()) );
    connect(notification, SIGNAL(closed()), this , SLOT(notifyclosed()) );  
}

void plasmoid_for_eBay_item::PlaceBidClickedFromNotify()
{
  notify_showes=false;
//  notification= new KNotification ( "lesswhen5min", KNotification::Persistant);
  notification= new KNotification ( "lesswhen5min");
  notification->setFlags(KNotification::Persistant);
  connect(notification, SIGNAL(activated(unsigned int )), this , SLOT(PlaceBidClickedFromNotify()) );
  connect(notification, SIGNAL(closed()), this , SLOT(notifyclosed()) );  
  PlaceBidClicked();
}


#include "plasmoid_for_ebay_item.moc"
