/*
    Copyright (c) 2010, Nick Sotnichek
    All rights reserved.

    Redistribution and use in source and binary forms, with or without
    modification, are permitted provided that the following conditions are met:
        * Redistributions of source code must retain the above copyright
        notice, this list of conditions and the following disclaimer.
        * Redistributions in binary form must reproduce the above copyright
        notice, this list of conditions and the following disclaimer in the
        documentation and/or other materials provided with the distribution.
        * Neither the name of the author nor the
        names of its contributors may be used to endorse or promote products
        derived from this software without specific prior written permission.

    THIS SOFTWARE IS PROVIDED BY NICK SOTNICHEK ''AS IS'' AND ANY
    EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
    WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
    DISCLAIMED. IN NO EVENT SHALL NICK SOTNICHEK BE LIABLE FOR ANY
    DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
    (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
    LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
    ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*/

#ifndef ITEM_PLASMOID_FOR_EBAY_HEADER
#define ITEM_PLASMOID_FOR_EBAY_HEADER

#include <KIcon>
#include <KNotification>
#include <kconfig.h>
#include <QLabel>
#include <QString>
#include <QAction>
#include <QTimer>
#include <QGraphicsSceneMouseEvent>
#include <QNetworkAccessManager>
#include <QNetworkReply>
#include <QKeyEvent>
#include <QNetworkProxy>

#include <Plasma/Applet>
#include <Plasma/Svg>
#include <plasma/widgets/lineedit.h>
#include <plasma/widgets/pushbutton.h>
#include <plasma/widgets/label.h>
#include <plasma/widgets/frame.h>

#include <urllabel.h>
#include <countryselector.h>

#include <global_defines.h>

class QSizeF;

struct item_data {
    QUrl apiUrl;
    QNetworkAccessManager *manager; // менеджер для загрузки информации о item 
    QNetworkAccessManager *picmanager; // менеджер для загрузки картинки item   
    QString s_result;  // строка статуса
    QString s_itemnumber; // номер item
    QString s_url; // url страницы item
    QString s_title; // заголовок
    QString s_bidcount; // кол-во ставок
    QString s_currentprice; // текущ цена
    QString s_currentprice_d; // текущ цена в числах
    QString s_currency; //Валюта цены
    QString s_listingtype; // тип листинга FixedPriceItem по фикс цене
    QString s_listingstatus; // статус итема
    QString s_condition; // состояние
    QString s_shprice; // цена доставки
    QString s_shprice_d; //цена доставки в числах
    QString s_shcurr; // Валюта цены доставки    
    QString s_cinterval; // если не равен 0 то значение интервала отображать красным
    QString s_sinterval; // интарвал для вывода на экран    
    QString s_picurl; // url картинки
    QString s_totalprice; //Полная цена с доставкой
    QString s_isnewversion; //Нужно ли обновить версию 1-нужно
    QString s_daysleft; // осталось дней
    QString s_hoursleft; //осталось часов
    QString s_minutesleft; //осталось минут
    QString s_secondsleft; //осталось секунд
    QString s_quantity; //кол-во в наличии по фикс цене
    QString s_country; //   страна доставки
    QString s_postalcode; // Индекс
    QString s_rcurrency;  //Валюта аукциона
    QString s_highbidder;  //ID пользователя текущего хай биддера
    QImage pic; // Картинка
    QImage flag; //Флаг
    QImage sigma; //Знак суммы
    QString s_orig_site; //Имя хоста сайта из URL паста
    QString s_site; //Имя хоста сайта где опубликован аукцион
};



class plasmoid_for_eBay_item : public Plasma::Applet
{
    Q_OBJECT
    public:

        plasmoid_for_eBay_item(QObject *parent, const QVariantList &args);
        ~plasmoid_for_eBay_item();


        void paintInterface(QPainter *painter,
                const QStyleOptionGraphicsItem *option,
                const QRect& contentsRect);
        void init();
	
	
     public slots:
        void updateItemStatus();
	void urlReturnStatus();
	void timetick();
	void timertick3();
	void timertick4();
	void selectcountry();
	void selectedcountry(QString cs, QString ps);
	void cancelcountry();
	void notifyclosed();
	
     private slots:
       void finished( QNetworkReply *reply );
       void finishedpic( QNetworkReply *reply );
       void PlaceBidClicked();
       void PlaceBidClickedFromNotify();
       void p_homepage();
       void p_paste();

       
     private:       
        Plasma::Svg m_svg;
        QTimer *m_timer1;
	QTimer *m_timer2;
	QTimer *m_timer3;
	QTimer *m_timer4;
	QImage m_image;
	QFont m_font;
	int state;
	int url_width;
	int url_height;
	int url_left;
	int url_top;
	int fl_x;
	int fl_y;
	int s_x;
	int s_y;
	int xxx,yyy;
	QString configGroup;
	QString item_num;
	char m_text[255];
	item_data m_d;
	int dcount;
	bool fl_notify;
	bool notify_showes;
	
	Plasma::Label *label;
	Plasma::Label *label_2;
	Plasma::Label *label_3;
	Plasma::Label *label_4;
	Plasma::Label *label_5;
	Plasma::Label *label_5a;	
	Plasma::Label *label_6;
	Plasma::Label *label_6a;	

	Plasma::Label *label_bids;
	Plasma::Label *label_timeleft;
	Plasma::Label *label_timeleftb;
	Plasma::Label *label_currbid;
	Plasma::Label *label_shipping;
	Plasma::PushButton *PlaceBidButton;
	Plasma::Label *label_totalprice;
	Plasma::Label *label_pic;
	int is_item_fl;	

	UrlLabel *applet_url;
	UrlLabel *applet_paste;
	UrlLabel *applet_but;
	UrlLabel *applet_flag;
	CountrySelector *cs_dialog;
	KNotification *notification;
	
	void dragEnterEvent(QGraphicsSceneDragDropEvent *event);
	void dropEvent(QGraphicsSceneDragDropEvent *event);
	void dropOrPasteEvent();	
	void dragLeaveEvent(QGraphicsSceneDragDropEvent *event);
	void keyPressEvent(QKeyEvent * event);
	bool processPhpResult(QIODevice *source);
	void LabelsON();
	void LabelsOFF();
	void ClearItem();
	void setstate_30();
	void setstate_25();
	void loadcountryandpostal();
	void savecountryandpostal();
	void changenotify();
	void clearoldconfig();
	void setProxy();
	bool setProxyFromString(QString s_proxy);
	void connectionFailed();
	void createNewConnectionManager();
	void dropOldConnectionManager();	
	
	QString moveState();
	void saveMyState();
	void removeConfig(QString configgrp);
	
	QString dstr; //debug string
	QNetworkProxy qproxy; //proxy for http connection
	int proxySource; //0 - noProxy, 1 - http_proxy, 2 - http_proxy with HostNameLookupCapability, 3 - KProtocolManager; 4-KProtocolManager -with HostNameLookupCapability; 5 -systemProxyForQuery
	QNetworkReply *qnreply;
};
 

K_EXPORT_PLASMA_APPLET(plasmoid_for_ebay_item, plasmoid_for_eBay_item)
#endif
