/*
    Copyright (c) 2010, Nick Sotnichek
    All rights reserved.

    Redistribution and use in source and binary forms, with or without
    modification, are permitted provided that the following conditions are met:
        * Redistributions of source code must retain the above copyright
        notice, this list of conditions and the following disclaimer.
        * Redistributions in binary form must reproduce the above copyright
        notice, this list of conditions and the following disclaimer in the
        documentation and/or other materials provided with the distribution.
        * Neither the name of the author nor the
        names of its contributors may be used to endorse or promote products
        derived from this software without specific prior written permission.

    THIS SOFTWARE IS PROVIDED BY NICK SOTNICHEK ''AS IS'' AND ANY
    EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
    WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
    DISCLAIMED. IN NO EVENT SHALL NICK SOTNICHEK BE LIABLE FOR ANY
    DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
    (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES;
    LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
    ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT
    (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
    SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.

*/
#include "urllabel.h"
#include <QGraphicsLinearLayout>
#include <QDesktopServices>
#include <QCursor>


	UrlLabel::UrlLabel(QGraphicsWidget *parent, bool isimage) : QObject() 
	{
		this->isimage=isimage;
		this->parent = parent;
		url=A_PROGRAM_URL;
		url_enabled=url_visible=true;
		initFrame();
	}

	UrlLabel::~UrlLabel() 
	{
	    delete(mFrame);
	    delete(mLabel);
	}

	Plasma::Frame *UrlLabel::frame() 
	{
		return mFrame;
	}

	Plasma::Label *UrlLabel::label() 
	{
		return mLabel;
	}


	void UrlLabel::setEnabled(bool enabled) 
	{
		mFrame->setEnabled(enabled);
		if(!isimage)
		  mLabel->setEnabled(enabled);
		url_enabled = enabled;
		if(!enabled)
		{
		  if(!isimage)
		      mFrame->setOpacity(0.01);
		  else
		      mFrame->setOpacity(0.2);
		}
	}
	
	void UrlLabel::setVisible(bool visible) 
	{
		mFrame->setVisible(visible);
		if(!isimage)
		  mLabel->setVisible(visible);
		url_visible = visible;
	}	
	
	void UrlLabel::setImage(QString ss)
	{	    
	    mFrame->setImage(ss);  
	}
	
	void UrlLabel::setText(QString ss)
	{
		mLabel->setText(ss);
	}

	void UrlLabel::setToolTip(QString ss)
	{
		mFrame->setToolTip(ss);
	}

	void UrlLabel::setGeomerty(QRect rect)
	{
	    mFrame->setGeometry(rect);
	    mFrame->setMinimumHeight(5);
	    mFrame->setGeometry(rect);	    
	    mLabel->setGeometry(rect);	    
	}

void UrlLabel::setCursor(QCursor cur)
{
    mFrame->setCursor(cur);
}


	void UrlLabel::initFrame() 
	{
		mFrame = new Plasma::Frame(parent);
		mFrame->setZValue(10);
		mLabel = new Plasma::Label(parent);
		mLabel->setAlignment(Qt::AlignHCenter|Qt::AlignCenter);
	        QFont font=QFont("Helvetica [Cronyx]");
		font.setItalic(true);
		font.setUnderline(true);
		font.setStretch(100);		
		mLabel->setFont(font);
		mLabel->setStyleSheet("color: rgb(0, 0, 150);");
		mFrame->setCursor(Qt::PointingHandCursor);
		mFrame->setOpacity(0.01);;
		mFrame->installEventFilter(this);
		mFrame->setAcceptHoverEvents(true);
		mFrame->setAcceptedMouseButtons(Qt::LeftButton);
		mFrame->setFrameShadow(Plasma::Frame::Raised);
		if(isimage)
		{
		  mLabel->setVisible(false);
		  mFrame->setOpacity(0.2);
		}
	}

	bool UrlLabel::eventFilter(QObject *obj, QEvent *event) 
	{
		if (obj == mFrame && url_enabled) 
		{
			if (event->type() == QEvent::GraphicsSceneHoverEnter) 
			{
			    mLabel->setStyleSheet("color: rgb(0, 0, 255);");
			    if(!isimage)
			      mFrame->setOpacity(0.2);
			    else
			      mFrame->setOpacity(0.7);
			} 
			else if (event->type() == QEvent::GraphicsSceneHoverLeave) 
			{			    
			    mLabel->setStyleSheet("color: rgb(0, 0, 150);");
			    if(!isimage)
			      mFrame->setOpacity(0.01);
			    else
			      mFrame->setOpacity(0.2);
			}
			else if (event->type() == QEvent::GraphicsSceneMousePress) 
			{
			    emit clicked();
			}			  
		}
		return false;
	}

#include "urllabel.moc"

